<?php

namespace Teknisa\SM\Service;

use Teknisa\SM\Factory\Sm00006001VariacaoTabelaPrecoServicoFactory as variacaoPrecBServFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;

class Sm00006001VariacaoTabelaPrecoServicoService {
    
    protected $variacaoPrecBServFactory;
    protected $Environment;
    
    
    public function __construct(variacaoPrecBServFactory $variacaoPrecBServFactory, Environment $Environment, NewCode $newCode) {
        
        $this->variacaoPrecBServFactory = $variacaoPrecBServFactory;
        $this->Environment  = $Environment;
        $this->newCode   = $newCode;
        
    }
    
     /**
     * A função "validaTipoMedidaVariavel" verifica se o tipo de medida variável
     * ja está criado para a faixa existente.
     */
     
    public function validaTipoMedidaVariavel($nrSeqItPrecBServ, $cdTpMedidaServ, $CdtpMedidaVariavel, $nrVaricacaoPrecBaseServ) {
        $params = array(
            "NRSEQITPRECBSERV" => $nrSeqItPrecBServ,
            "CDTPMEDIDASERV" => $cdTpMedidaServ,
            "CDTPMEDIDAVARIAVEL" => $CdtpMedidaVariavel,
            "NRVARIACAOPRECBASESERV" => $nrVaricacaoPrecBaseServ,
            "NRORG"            => $this->Environment->getNrOrg()
        );
        
        return $this->variacaoPrecBServFactory->validaTipoMedidaVariavel($params);
    
    }
    
    
     /**
     * A função "insereVariacaoPreco" insere os valores de variação de preço 
     * ao inserir um novo tipo de medida na variação.
     */
     
    public function insereVariacaoPreco($nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel, $vrPrecoVariacao, $idMultipVariacao, $qtFranquia, $sincronizar, $usaFranquia){
       $nrorg = $this->Environment->getNrOrg();
       $operador =$this->Environment->getCdOperador(); 
       
       /* Sincronizar faixas de preço*/
       if ($sincronizar == true){
          /* recebe as faixas a serem inseridas */
          
          $paramsSinc = array(
           
                "CDTPMEDIDASERV"          => $cdTpMedidaServ, 
                "CDTPMEDIDAVARIAVEL"      => $cdtpMedidaVariavel, 
                "NRORG"                   => $nrorg
          );
          
          $faixas = $this->variacaoPrecBServFactory->sinconizaMedidaVariavel($paramsSinc);
          
          if (count($faixas) == 0){
              
             /* Se não achar nenhuma faixa para a variação informada, insere a faixa padrão */
             
             $this->geraVarPadrao($nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel, $vrPrecoVariacao, $idMultipVariacao, $qtFranquia, $nrorg, $operador, $usaFranquia);
            
          }
          else{
              /* Se achar faixa(s) para a variação informada, insere a(s) faixa(s) encontrada(s) */
              
              $countFranquia = 0;
              
              foreach ($faixas as $faixa) {
               
                 $qtIniVariacao = $faixa['QTMEDIDAINIVARIACAO'];
                 $qtFimVariacao = $faixa['QTMEDIDAFIMVARIACAO']; 
                 $percentual    = $faixa['VRPERCVARIACAO']; 
                 
                 if ($usaFranquia == 'S'){
                    
                    $validaFranquia = $this->geraFaixasFranquia($nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel, $qtIniVariacao, $qtFimVariacao, $vrPrecoVariacao, $percentual, $idMultipVariacao, $qtFranquia, $nrorg, $operador, $countFranquia);
                    
                    $countFranquia = $countFranquia+1;
                 }
                 else{
                     
                    $validaFranquia = $this->geraFaixas($nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel, $qtIniVariacao, $qtFimVariacao, $vrPrecoVariacao, $percentual, $idMultipVariacao, $qtFranquia, $nrorg, $operador);
                 };    
              }
          }
           
       }
       /* NÃO Sincronizar faixas de preço* - insere a faixa padrão */
       else{
            $this->geraVarPadrao($nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel, $vrPrecoVariacao, $idMultipVariacao, $qtFranquia, $nrorg, $operador, $usaFranquia);
       }
       
       /*if ($usaFranquia == 'S'){
           $this->replicaFranquia($nrSeqItPrecBServ, $cdtpMedidaVariavel, $qtFranquia);
       }*/
    }
    
    /**
     * A função "editaVariacaoPreco" edita os valores de variação de preço
     * ao alterar um tipo de medida na variação.
     */
     
    public function editaVariacaoPreco($idMultipVariacao, $nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel, $qtFranquia, $usaFranquia){
        $nrorg = $this->Environment->getNrOrg();
        $operador = $this->Environment->getCdOperador();
       
        $params = array(
            "IDMULTIPVARIACAO"    => $idMultipVariacao,
            "OPERADOR"            => $operador,
            "NRORG"               => $nrorg,
            "NRSEQITPRECBSERV"    => $nrSeqItPrecBServ,
            "CDTPMEDIDASERV"      => $cdTpMedidaServ,
            "CDTPMEDIDAVARIAVEL"  => $cdtpMedidaVariavel,
            "QTFRANQUIA"          => $qtFranquia
        );
        
        $return = $this->variacaoPrecBServFactory->editaVariacaoPreco($params);
        
        if ($usaFranquia == 'S') {
           $this->geraVarPadrao($nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel, 0, $idMultipVariacao, $qtFranquia, $nrorg, $operador, $usaFranquia);
           //$this->replicaFranquia($nrSeqItPrecBServ, $cdtpMedidaVariavel, $qtFranquia);
        }
    }
    
    
    /**
     * A função "deletaVariacaoPreco" insere os valores de variação de preço 
     * ao excluir um tipo de medida na variação.
     */
     
    public function deletaVariacaoPreco($nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel){
    
        $params = array(
            "NRORG"               => $this->Environment->getNrOrg(),
            "NRSEQITPRECBSERV"    => $nrSeqItPrecBServ,
            "CDTPMEDIDASERV"      => $cdTpMedidaServ,
            "CDTPMEDIDAVARIAVEL"  => $cdtpMedidaVariavel
        );
        
        $return = $this->variacaoPrecBServFactory->deletaVariacaoPreco($params);
        
    }
    
    
    /**
     *  A função "validaFranquiaFaixa" verifica se existem faixas de variação cuja 
     *  franquia esteja dentro dela ou que seja menor do que a franquia.
     */
     
    public function validaFranquiaFaixa($nrSeqItPrecBServ, $cdTpMedidaServ, $CdtpMedidaVariavel, $qtFranquia) {
        $params = array(
            "NRSEQITPRECBSERV"   => $nrSeqItPrecBServ,
            "CDTPMEDIDASERV"     => $cdTpMedidaServ,
            "CDTPMEDIDAVARIAVEL" => $CdtpMedidaVariavel,
            "QTFRANQUIA"         => $qtFranquia,
            "NRORG"              => $this->Environment->getNrOrg()
        );
        
        return $this->variacaoPrecBServFactory->validaFranquiaFaixa($params);
    }


    /* Nesta função é gerada a variação de preço padrão da medida variável */
    
    public function geraVarPadrao($nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel, $vrPrecoVariacao, $idMultipVariacao, $qtFranquia, $nrorg, $operador, $usaFranquia){
        
         
        if ($usaFranquia == 'S'){
                
            $qtmedidafim = $qtFranquia;
            $vrPreco = 0;
        }
        else{
            
            $qtmedidafim = 99999999;
            $vrPreco = $vrPrecoVariacao;
        };
         
        $novoCodigo = $this->newCode->getCodeByTableName("FCT_VARIACAOPRECBASESERV", $nrorg, 1, 10, true);
        
        $params = array(
             "NRVARIACAOPRECBASESERV"  => $novoCodigo,  
             "NRSEQITPRECBSERV"        => $nrSeqItPrecBServ,  
             "CDTPMEDIDASERV"          => $cdTpMedidaServ, 
             "CDTPMEDIDAVARIAVEL"      => $cdtpMedidaVariavel, 
             "QTMEDIDAINIVARIACAO"     => 0,
             "QTMEDIDAFIMVARIACAO"     => $qtmedidafim, 
             "VRPRECOVARIACAO"         => $vrPreco,
             "IDMULTIPVARIACAO"        => $idMultipVariacao,
             "QTFRANQUIA"              => $qtFranquia,
             "NRORG"                   => $nrorg,
             "OPERADOR"                => $operador
        );
         
        $return = $this->variacaoPrecBServFactory->insereVariacaoPreco($params); 
    }
    
    
    /* Nesta função são geradas as variações de preço da medida variável. */
    
    public function geraVariacao($nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel, $qtIniVariacao,  $qtFimVariacao, $vrPrecoVariacao, $idMultipVariacao, $qtFranquia, $nrorg, $operador){
        
         
        $novoCodigo = $this->newCode->getCodeByTableName("FCT_VARIACAOPRECBASESERV", $nrorg, 1, 10, true);
        
        $params = array(
             "NRVARIACAOPRECBASESERV"  => $novoCodigo,  
             "NRSEQITPRECBSERV"        => $nrSeqItPrecBServ,  
             "CDTPMEDIDASERV"          => $cdTpMedidaServ, 
             "CDTPMEDIDAVARIAVEL"      => $cdtpMedidaVariavel, 
             "QTMEDIDAINIVARIACAO"     => $qtIniVariacao,
             "QTMEDIDAFIMVARIACAO"     => $qtFimVariacao, 
             "VRPRECOVARIACAO"         => $vrPrecoVariacao,
             "IDMULTIPVARIACAO"        => $idMultipVariacao,
             "QTFRANQUIA"              => $qtFranquia,
             "NRORG"                   => $nrorg,
             "OPERADOR"                => $operador
        );
         
        $return = $this->variacaoPrecBServFactory->insereVariacaoPreco($params); 
        
    }
    
    
    /* Nesta função são geradas as faixas de preço da media variável definida no
       tipo de medida. Considerando a franquia definida. */
       
    public function geraFaixasFranquia($nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel, $qtIniVariacao, $qtFimVariacao, $vrPrecoVariacao, $percentual, $idMultipVariacao, $qtFranquia, $nrorg, $operador, $countFranquia){
        
        if ($countFranquia == 0){
            
            $this->geraVarPadrao($nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel, 0, $idMultipVariacao, $qtFranquia, $nrorg, $operador, 'S');
        }
            
        if(($qtIniVariacao <= $qtFranquia) && ($qtFimVariacao > $qtFranquia)) {
            
            $qtIniVariacaoAux =  $qtFranquia+0.01;
            
            $this->geraFaixas($nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel, $qtIniVariacaoAux, $qtFimVariacao, $vrPrecoVariacao, $percentual, $idMultipVariacao, $qtFranquia, $nrorg, $operador); 
                
        }
        else if(($qtIniVariacao > $qtFranquia) && ($qtFimVariacao > $qtFranquia)) {
            
            $this->geraFaixas($nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel, $qtIniVariacao, $qtFimVariacao, $vrPrecoVariacao, $percentual, $idMultipVariacao, $qtFranquia, $nrorg, $operador);
        }
    }
    
    
    /* Nesta função são geradas as faixas de preço da media variável definida no
       tipo de medida. */
    
    public function geraFaixas($nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel, $qtIniVariacao, $qtFimVariacao, $vrPrecoVariacao, $percentual, $idMultipVariacao, $qtFranquia, $nrorg, $operador){
        
        if ($percentual > 0){
                     
            $precovariacaocalc = ($vrPrecoVariacao*$percentual)/100;    
            $precovariacao     = $vrPrecoVariacao - $precovariacaocalc;     
        }
         
        if ($percentual < 0){
            $precovariacaocalc = $percentual* -1;
            $precovariacaocalc = ($vrPrecoVariacao*$precovariacaocalc)/100;    
            $precovariacao     = $vrPrecoVariacao + $precovariacaocalc;    
        }
         
        if ($percentual == 0){
             
             $precovariacao = $vrPrecoVariacao;
        }
    
        $this->geraVariacao($nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel, $qtIniVariacao, $qtFimVariacao, $precovariacao, $idMultipVariacao, $qtFranquia, $nrorg, $operador);
    }


    /* Nesta função é realizada a replicação da franquia para a medida variavel dos
       demais itens da proposta.*/
    
    /*public function replicaFranquia($nrSeqItPrecBServ, $cdtpMedidaVariavel, $qtFranquia){
        
        $nrorg = $this->Environment->getNrOrg();
        $operador =$this->Environment->getCdOperador(); 
        
        $dadosReplicacao = $this->buscaDadosReplicacao($nrSeqItPrecBServ, $cdtpMedidaVariavel, $nrorg);
        
        if($dadosReplicacao != null) {
        
            $nrSeqItPrecBSerAnte = 0;
    
            foreach ($dadosReplicacao as $replicacao) {
                
                $nrSeqItPrecBSerAux    = $replicacao['NRSEQITPRECBSERV'];
                $cdTpMedidaServAux     = $replicacao['CDTPMEDIDASERV'];
                $cdtpMedidaVariavelAux = $replicacao['CDTPMEDIDAVARIAVEL'];
                $qtIniVariacaoAux      = $replicacao['QTMEDIDAINIVARIACAO'];
                $qtFimVariacaoAux      = $replicacao['QTMEDIDAFIMVARIACAO'];
                $precovariacaoAux      = $replicacao['VRPRECOVARIACAO'];
                $idMultipVariacaoAux   = $replicacao['IDMULTIPVARIACAO'];
    
                if ($nrSeqItPrecBSerAux == $nrSeqItPrecBSerAnte){
                    
                    $this->geraFaixasFranquiaRep($nrSeqItPrecBSerAux, $cdTpMedidaServAux, $cdtpMedidaVariavelAux, $qtIniVariacaoAux, $qtFimVariacaoAux, $precovariacaoAux, $idMultipVariacaoAux, $qtFranquia, $nrorg, $operador, 'N');
                    
                    $nrSeqItPrecBSerAnte = $nrSeqItPrecBSerAux;
                    
                }
                else{
                    $this->deletaVariacaoPreco($nrSeqItPrecBSerAux, $cdTpMedidaServAux, $cdtpMedidaVariavelAux);
                    
                    $this->geraFaixasFranquiaRep($nrSeqItPrecBSerAux, $cdTpMedidaServAux, $cdtpMedidaVariavelAux, $qtIniVariacaoAux, $qtFimVariacaoAux, $precovariacaoAux, $idMultipVariacaoAux, $qtFranquia, $nrorg, $operador, 'S');
                     
                    $nrSeqItPrecBSerAnte = $nrSeqItPrecBSerAux;
                    
                }
            }
        }
    }*/
    
    
    /* Nesta função são buscadas as faixas de preço atuais da medida variável para 
       realziação da replicação da franquia  */
       
    public function buscaDadosReplicacao($nrSeqItPrecBServ, $cdtpMedidaVariavel, $nrorg){
        
        $params = array(
             "NRSEQITPRECBSERV"        => $nrSeqItPrecBServ,  
             "CDTPMEDIDAVARIAVEL"      => $cdtpMedidaVariavel, 
             "NRORG"                   => $nrorg
        );
         
        $return = $this->variacaoPrecBServFactory->buscaDadosReplicacao($params);
        
        return $return;
        
    }
    
    
    /* Nesta função são geraas as faixas de preço de replicação quanto utilizado franquia. */
    
    public function geraFaixasFranquiaRep($nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel, $qtIniVariacao, $qtFimVariacao, $precovariacao, $idMultipVariacao, $qtFranquia, $nrorg, $operador, $geraVarPadrao){
        
        if ($geraVarPadrao == 'S'){
            
            $this->geraVarPadrao($nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel, 0, $idMultipVariacao, $qtFranquia, $nrorg, $operador, 'S');
        }
            
        if(($qtIniVariacao <= $qtFranquia) && ($qtFimVariacao > $qtFranquia)) {
            
            $qtIniVariacaoAux =  $qtFranquia+0.01;
            
            $this->geraVariacao($nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel, $qtIniVariacaoAux, $qtFimVariacao, $precovariacao, $idMultipVariacao, $qtFranquia, $nrorg, $operador); 
                
        }
        else if(($qtIniVariacao > $qtFranquia) && ($qtFimVariacao > $qtFranquia)) {
            
            $this->geraVariacao($nrSeqItPrecBServ, $cdTpMedidaServ, $cdtpMedidaVariavel,$qtIniVariacao,  $qtFimVariacao, $precovariacao, $idMultipVariacao, $qtFranquia, $nrorg, $operador);
        }
    
    }
    
    
}

?>